@@ -1,11 +1,14 @@
 import os
-from copper.io.io import *
+from copper.core.io import *
 from copper.core.set import *
 from copper.core.ml import *
+import copper.core.r as r
+
 import copper.viz.base as plot
 
-import copper.core.transforms as transform
-import copper.core.r as r
+import copper.utils.transforms as transform
+import copper.utils.frame
+import copper.utils.ml
 
 from copper.core.config import Project
 project = Project()
